### Ethernet repeater hub simulator (UDP w/o select) ###
#
# Public domain software
# (When using Python 2, part of this program must be commented/uncommented)
#
# Coded by Yasusi Kanada
# 2015-5-18, 6-21; 2016-6-25

import optparse, socket, time
from ether import SMAC, DMAC, printPacket


BUF_SIZE = 3100		# > 2 * 1500

host = socket.gethostbyname(socket.gethostname())


### Packet handler ###

def sendPacket(packet, destPort):
    sock, addr = vport2remote[destPort]
    return sock.sendto(packet, addr) // 2

def printForwarding(message, dmac):
    print("{} {} {} {} {}".format("@@@", message, "- DMAC:", dmac, "@@@"))

def printSending(sendBytes, destPort):
    print("{} {} {}".format(sendBytes, "bytes sent through port", destPort))


### Packet forwarding ###

def forward(packet, sourcePort, ports):
    smac = SMAC(packet)
    now = time.time()
    if opts.monitorLevel >= 1:
        printPacket(packet, now, "Received:")

    # Forward packet
    dmac = DMAC(packet)
    if opts.monitorLevel >= 1:
        printForwarding("Flood packet", dmac)
    for destPort in ports:
        if sourcePort != destPort:
            sendBytes = sendPacket(packet, destPort)
            if opts.monitorLevel >= 2:
                printSending(sendBytes, destPort)


### Main program ###

def main():
    global opts, vport2remote, host

    # Parse command line
    parser = optparse.OptionParser()
    for i in range(0, 8):
        str_i = str(i);
        parser.add_option("--lp" + str_i, "--lport" + str_i,
                          dest = "lport" + str_i, type = "int",
                          help = ("Local (my) network port id for link " + str_i))
        parser.add_option("--rp" + str_i, "--rport" + str_i,
                          dest = "rport" + str_i, type = "int",
                          help = ("Remote (next) network port id for link " + str_i))
    parser.add_option("--nports", dest = "nports", type = "int",
                      help = ("Number of local network ports"))
    parser.add_option("--timeout", dest = "timeout", type = "int",
                      help = "Learning timeout time")
    parser.add_option("--monitorLevel", "--monitor",
                      dest = "monitorLevel", type="int",
                      help = ("Monotoring level (0: Silent, 1: Sending/Receiving 2: Debug)"))
    parser.add_option("--dumpMAC", dest = "dumpMAC", action = "store_true")
    parser.set_defaults(lport0 = 4000, lport1 = 4001, lport2 = 4002, lport3 = 4003,
                        lport4 = 4004, lport5 = 4005, lport6 = 4006, lport7 = 4007,
                        rport0 = 5000, rport1 = 5001, rport2 = 5002, rport3 = 5003,
                        rport4 = 5004, rport5 = 5005, rport6 = 5006, rport7 = 5007,
                        nports = 8, timeout = 300, monitorLevel = 0)
    opts, args = parser.parse_args()

    # Create port lists
    lport = [opts.lport0, opts.lport1, opts.lport2, opts.lport3,
             opts.lport4, opts.lport5, opts.lport6, opts.lport7]

    rport = [opts.rport0, opts.rport1, opts.rport2, opts.rport3,
             opts.rport4, opts.rport5, opts.rport6, opts.rport7]

    # Open sockets
    sock = list([None] * opts.nports)
    sockPort = {}
    for i in range(0, opts.nports):
        sock[i] = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock[i].setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock[i].setblocking(0)
        sock[i].bind((host, lport[i]))
        sockPort[sock[i]] = lport[i]

    # Create link info (virtual port to remote address)
    vport = list([None] * opts.nports)
    vport2remote = {}
    for i in range(0, opts.nports):
        vport[i] = lport[i]
        vport2remote[vport[i]] = (sock[i], (host, rport[i]))

    # Repeat forwarding a packet
    interval = 1
    lastTime = time.time()
    while True:
        for i in range(0, opts.nports):
            sourceSock = sock[i]
            try:
                packet = sourceSock.recv(BUF_SIZE)
            except socket.error:
                pass
            else:
                forward(packet, sockPort[sourceSock], vport)

        now = time.time()
        if now > lastTime + interval:
            lastTime = now
        else:
            time.sleep(0.001001)

main()
