### Ethernet terminal simulator (UDP w/o select) ###
#
# Public domain software
# (When using Python 2, part of this program must be commented/uncommented)
#
# Coded by Yasusi Kanada
# 2013-5-19, 24, 6-10; 2014-5-25, 6-8; 2015-5-10, 6-21

import optparse, socket, time
from ether import DMAC, printPacket


BUF_SIZE = 3100		# > 2 * 1500

host = socket.gethostbyname(socket.gethostname())


### Main program ###

def terminal():
    # Parse command line
    parser = optparse.OptionParser()
    parser.add_option("--lp", "--lport", "--localPort", dest = "lport", type="int",
                      help = "Local network port id")
    parser.add_option("--rp", "--rport", "--remotePort", dest = "rport", type="int",
                      help = "Remote (next) network port id")
    parser.add_option("--lm", "--lmac", "--localMAC", dest = "lmac", type = "str",
                      help = "Local (my) MAC address")
    parser.add_option("--rm", "--rmac", "--remoteMAC", dest = "rmac", type = "str",
                      help = "Remote (receiver) MAC address")
    parser.add_option("--promiscuous", dest = "promiscuous", action = "store_true",
                      help = "Promiscuous mode")
    parser.add_option("--receiveOnly", dest = "receiveOnly", action = "store_true")
    parser.set_defaults(lport = 4000, rport = 5000, promiscuous = 0,
                        lmac = "000000000000", rmac = "000000000000")
    opts, args = parser.parse_args()

    # Open socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.setblocking(0)
    sock.bind((host, opts.lport))

    # Contents of packet to send (constant)
    sendPacketString = opts.rmac + opts.lmac + "0800" + \
        "000102030405060708090a0b0c0d0e0f"
    sendPacket = bytes(sendPacketString, 'utf-8')		# !! Python 3 !!
    # sendPacket = sendPacketString				# !! Python 2 !!

    # Repeat sending and receiving packets
    interval = 1
    lastTime = time.time();
    while True:
        now = time.time()

        try:
            packet = sock.recv(BUF_SIZE)
        except socket.error:
            pass
        else:
            dmac = DMAC(packet)
            if opts.promiscuous or dmac == opts.lmac:
                printPacket(packet, now, "Received:")

        if not opts.receiveOnly:
            if now > lastTime + interval:
                sendBytes = sock.sendto(sendPacket, (host, opts.rport))
                printPacket(sendPacket, now, "Sent:    ")
                lastTime = now
            else:
                time.sleep(0.001001)
        else:
            time.sleep(0.001001)

terminal()
